
static char help[] = "Matrix Partitioning Without Spectral Graph Partitioning By Using ParMETIS.\n\n";
  
#include <petscmat.h>   

#include <petscis.h> 

#include <petscsys.h>

#include <stdio.h>

#include <stdlib.h>   

#include <petsctime.h>


PetscErrorCode absmat(Mat A,Mat *AbsA)                                              
{
  PetscErrorCode ierr;
  PetscInt       n,i,nc,j,rstart,rend;
  const PetscInt *aj;                                                                
  const PetscScalar *aa;
  PetscScalar *absaa; 
  
  ierr = MatGetSize(A,&n,NULL);CHKERRQ(ierr);
  ierr = MatDuplicate(A,MAT_COPY_VALUES,AbsA);CHKERRQ(ierr);
       
  
  /* copy over the matrix entries from the matrix A to matrix AbsA*/
  ierr = MatGetOwnershipRange(A,&rstart,&rend);CHKERRQ(ierr);                  
  

  for (i=rstart; i<rend; i++) {                                                 
    ierr = MatGetRow(A,i,&nc,&aj,&aa);CHKERRQ(ierr);                               
    
    /* replace the nonzero values with their absolute values*/
    
    PetscMalloc1(nc,&absaa);
    for (j=0; j<nc; j++){
        absaa[j] = fabs(aa[j]);
    }
    
    ierr = MatSetValues(*AbsA,1,&i,nc,aj,absaa,INSERT_VALUES);CHKERRQ(ierr);    
    ierr = MatRestoreRow(A,i,&nc,&aj,&aa);CHKERRQ(ierr);
  }
                    
  return(0);          
}


int main (int argc,char **argv)                                                     
{
                      
  Mat               A=NULL,AL;                                                       
  IS                partitioning;
  PetscViewer       fd; 
  char              file[PETSC_MAX_PATH_LEN];
  PetscBool         flg;  
  PetscErrorCode    ierr;
  MatPartitioning   part;


  //ierr = SlepcInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
  
  // Read matrix from Petsc Binary File                                          

  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
  
  /*
     Determine files from which we read matrix
  */
  
  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");

  /*
     Open binary file.  Note that we use FILE_MODE_READ to indicate
     reading from this file.
  */
  
  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);

  /*
    Load the matrix; then destroy the viewer.
  */
  
  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);  
    MatSetType(A,MATMPIAIJ); 
  ierr = MatSetOptionsPrefix(A,"a_");CHKERRQ(ierr);
  ierr = MatSetFromOptions(A);CHKERRQ(ierr);
  ierr = MatLoad(A,fd);CHKERRQ(ierr);
  ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
  
  
    /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                    Create Partitioning
     - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

      /* Symmetry check */
      
      Mat Atr,SymmA,Atrabs,Aabs;
      PetscBool isEqual;
      Vec D;
      PetscInt i;

    /*ierr = MatTranspose(A,MAT_INITIAL_MATRIX,&Atr);CHKERRQ(ierr);
    ierr = MatDuplicate(Atr,MAT_COPY_VALUES,&SymmA);CHKERRQ(ierr);*/
      flg = PETSC_TRUE;                                                           
    PetscOptionsGetBool(NULL,NULL, "-check_symmetry", &flg,NULL);
    if (flg) {
        MatIsSymmetric(A,0.0,&isEqual);
        if (isEqual) {
        ierr = PetscPrintf(PETSC_COMM_WORLD,"Input matrix is symmetric\n\n");CHKERRQ(ierr);
        ierr = absmat(A,&SymmA);CHKERRQ(ierr);

        } else {ierr = PetscPrintf(PETSC_COMM_WORLD,"Input matrix is not symmetric\n\n");CHKERRQ(ierr);
            ierr = MatTranspose(A,MAT_INITIAL_MATRIX,&Atr);CHKERRQ(ierr);
                ierr = absmat(Atr,&Atrabs);CHKERRQ(ierr);
                ierr = absmat(A,&Aabs);CHKERRQ(ierr);
                ierr = MatAssemblyBegin(Atrabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);      
                ierr = MatAssemblyEnd(Atrabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
                ierr = MatDuplicate(Atrabs,MAT_COPY_VALUES,&SymmA);CHKERRQ(ierr);
                ierr = MatAssemblyBegin(Aabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);      
                ierr = MatAssemblyEnd(Aabs,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
                ierr = MatAXPY(SymmA,1.,Aabs,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr); 
        }

    }
        ierr = MatAssemblyBegin(SymmA,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
        ierr = MatAssemblyEnd(SymmA,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
        
        
  ierr = MatConvert(SymmA,MATMPIADJ,MAT_INITIAL_MATRIX,&AL);CHKERRQ(ierr);                           

  ierr = MatPartitioningCreate(MPI_COMM_WORLD,&part);CHKERRQ(ierr);
  ierr = MatPartitioningSetAdjacency(part,AL);CHKERRQ(ierr); 
   ierr = MatPartitioningSetFromOptions(part);CHKERRQ(ierr);
   
  PetscBool use_edge_weights=PETSC_FALSE;
  PetscOptionsGetBool(NULL,NULL,"-test_use_edge_weights",&use_edge_weights,NULL);
  if (use_edge_weights) {
      MatPartitioningSetUseEdgeWeights(part,use_edge_weights);
      MatPartitioningGetUseEdgeWeights(part,&use_edge_weights);
      if (!use_edge_weights) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP, "use_edge_weights flag does not setup correctly \n");
    }
  
  ierr = MatPartitioningApply(part,&partitioning);CHKERRQ(ierr);
 ISView(partitioning,PETSC_VIEWER_STDOUT_WORLD);

  
  ierr = ISDestroy(&partitioning);CHKERRQ(ierr);
  ierr = MatPartitioningDestroy(&part);CHKERRQ(ierr);

 
  ierr = MatDestroy(&AL);CHKERRQ(ierr);
  
  ierr = MatDestroy(&A);CHKERRQ(ierr);                    
  
  ierr = PetscFinalize();

  return ierr;
} 


/*TEST

   build:
      requires:  !complex !define(PETSC_USE_64BIT_INDICES), parmetis

   test:
      args: -f ${wPETSC_DIR}/share/petsc/datafiles/matrices/binary_files/nos7_binary  -mat_partitioning_type parmetis
      
      example: mpirun -n 2 ./test_adj -f /home/edaoktay/petsc-3.13.2/share/petsc/datafiles/matrices/binary_files/Ragusa18_binary -mat_partitioning_type chaco -test_use_edge_weights 1


TEST*/

